/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.graph.impl;

import javax.annotation.Nonnull;
import java.awt.Font;
import org.netbeans.api.visual.border.Border;
import org.netbeans.api.visual.border.BorderFactory;
import org.netbeans.api.visual.layout.LayoutFactory;
import org.netbeans.api.visual.widget.LabelWidget;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;
import org.openide.util.Parameters;
import it.tidalwave.semantic.other.Entity;
import it.tidalwave.semantic.other.Property;
import java.util.Arrays;
import java.util.List;
import org.openide.nodes.Node;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class DefaultEntityWidget extends Widget
  {
    private static final String RDF_TYPE = "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";
    private static final String RDFS_LABEL = "http://www.w3.org/2000/01/rdf-schema#label";
    private static final Border BORDER_4 = BorderFactory.createLineBorder(4);

    @Nonnull
    private final Node node;
    
    private LabelWidget displayNameWidget;

    private LabelWidget typeWidget;

    private Widget propertiesWidget;

    private static final NamespaceManager nameSpaceManager = new NamespaceManager();
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public DefaultEntityWidget (@Nonnull final Scene scene,
                                @Nonnull final Node node)
      {
        super(scene);
        Parameters.notNull("node", node);
        this.node = node;
//        this.entity = entity;
        setLayout(LayoutFactory.createVerticalFlowLayout());
        setBorder(BorderFactory.createLineBorder());
        setOpaque(true);
        setCheckClipping(true);

//        final Widget auxWidget = new Widget(scene);
//        auxWidget.setLayout(LayoutFactory.createHorizontalFlowLayout());
//        auxWidget.setBorder(BORDER_4);
//        addChild(auxWidget);

//        ImageWidget classImage = new ImageWidget(scene);
//        classImage.setImage (IMAGE_CLASS);
//        classWidget.addChild (classImage);

        displayNameWidget = new LabelWidget(scene);
        displayNameWidget.setFont(scene.getDefaultFont().deriveFont(Font.BOLD));
        displayNameWidget.setLabel(node.getDisplayName());
        displayNameWidget.setBorder(BORDER_4);
        addChild(displayNameWidget);
        
        final Entity entity = node.getLookup().lookup(Entity.class);

        if (!entity.getType().getDisplayName().equals(""))
          {
            typeWidget = new LabelWidget(scene);
            typeWidget.setFont(scene.getDefaultFont().deriveFont(Font.ITALIC));
            typeWidget.setLabel(nameSpaceManager.resolveURI(entity.getType().getDisplayName()));
            typeWidget.setBorder(BORDER_4);
            addChild(typeWidget);
          }

        final List<String> ignoredPropertyTypes = Arrays.asList(RDF_TYPE, RDFS_LABEL);

        for (final Property<?> property : entity.getProperties())
          {
            if (!ignoredPropertyTypes.contains(property.getType().stringValue()))
              {
                addPropertyWidget(property);
              }
          }
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public void setDisplayNameVisible (final boolean visible)
      {
        displayNameWidget.setVisible(visible);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public boolean isDisplayNameVisible()
      {
        return displayNameWidget.isVisible();
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    private void addPropertyWidget (@Nonnull final Property<?> property)
      {
        final Scene scene = getScene();

        if (propertiesWidget == null)
          {
            propertiesWidget = new Widget(scene);
            propertiesWidget.setLayout(LayoutFactory.createVerticalFlowLayout());
            propertiesWidget.setOpaque(false);
            propertiesWidget.setBorder(BORDER_4);
            addChild(propertiesWidget);
          }

        final Widget widget = new Widget(scene);
        widget.setLayout(LayoutFactory.createHorizontalFlowLayout());

//        ImageWidget imageWidget = new ImageWidget (scene);
//        imageWidget.setImage (IMAGE_OPERATION);
//        widget.addChild (imageWidget);

        final LabelWidget labelWidget = new LabelWidget(scene);
        final String name = nameSpaceManager.resolveURI(property.getDisplayName());
        final String value = nameSpaceManager.resolveURI(property.get().toString());
        labelWidget.setLabel(name + " = " + value);
        widget.addChild(labelWidget);
        propertiesWidget.addChild(widget);
    }
  }
